"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildAggregate = void 0;
const lodash_clonedeep_1 = __importDefault(require("lodash.clonedeep"));
/**
 * Use this function to build your aggregate instances.
 * @param state The current state of you aggregate
 * @param resolver The EventResolver your aggregate will use to mutate the state.
 * @returns This function will return an new aggregate instance.
 * @example
 * ```typescript
 * export function createCounter(
 *   commandId: string,
 *   state?: CounterState
 * ): Counter {
 *   const initialState: CounterState = {
 *     id: "none",
 *     count: 0,
 *     sequence: 0,
 *     commandId: commandId,
 *   };
 *
 *   if (state) {
 *     state.commandId = commandId;
 *   }
 *
 *   const currentState = state || initialState;
 *
 *   const aggregate = buildAggregate(currentState, counterEventResolver);
 *
 *   return {
 *     init: buildInit(aggregate),
 *     count: buildCount(aggregate),
 *     reset: buildReset(aggregate),
 *     ...aggregate,
 *   };
 * }
 * ```
 */
function buildAggregate(state, resolver, options) {
    const history = {
        allEvents: [],
        uncommitedEvents: [],
        snapshots: [],
    };
    return {
        apply: (event) => apply(state, event, resolver, history),
        addEvent: (event) => addEvent(state, event, resolver, history, options),
        getSequence: () => state.sequence,
        getUncommmitedEvents: () => history.uncommitedEvents,
        eventsCommited: () => (history.uncommitedEvents = []),
        state: () => lodash_clonedeep_1.default(state),
        snapshot: () => {
            const snapshot = lodash_clonedeep_1.default(state);
            history.snapshots.push(snapshot);
            return snapshot;
        },
        getSnapshot: () => {
            return history.snapshots;
        },
        snapshotsCommited: () => {
            history.snapshots = [];
        },
    };
}
exports.buildAggregate = buildAggregate;
function apply(state, events, resolver, history) {
    const eventList = Array.isArray(events) ? events : [events];
    const successOutcomes = [];
    for (let i = 0; i < eventList.length; i++) {
        const event = eventList[i];
        if (event.sequence !== state.sequence + 1) {
            return {
                outcome: "FAILURE",
                errorCode: "APPLY_EVENT_FAILED",
                reason: "Incoherent sequence",
                data: {
                    event: event,
                    stateSequence: state.sequence,
                },
            };
        }
        try {
            const result = resolver(state, event);
            if (result.outcome === "FAILURE") {
                return result;
            }
            successOutcomes.push(result);
        }
        catch (error) {
            return {
                outcome: "FAILURE",
                errorCode: "APPLY_EVENT_FAILED",
                reason: "Resolver thrown an error",
                data: {
                    error: error.name,
                    message: error.message,
                    stack: error.stack,
                },
            };
        }
        history.allEvents.push(event);
        state.sequence += 1;
    }
    return {
        outcome: "SUCCESS",
        data: {
            results: successOutcomes,
        },
    };
}
function addEvent(state, event, resolver, history, options) {
    const snapshotEvery = options?.snapshotEvery;
    const result = apply(state, event, resolver, history);
    if (result.outcome === "FAILURE") {
        return result;
    }
    if (snapshotEvery && state.sequence % snapshotEvery === 0) {
        const snapshot = lodash_clonedeep_1.default(state);
        history.snapshots.push(snapshot);
    }
    history.uncommitedEvents.push(event);
    return result;
}
